//DEMINEUR V3.0 PAR EMERIC POUPON (aka ITM)

#define USE_TI89              // Produce .89z File
#define USE_TI92PLUS          // Produce .9xz File
#define USE_V200

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#define VERSION "v3.0"
#define VERSION_TIME "(28-01-2003)"
#define SAVE_NAME "minesf"

#define NONE 0
#define EASY 1
#define MEDIUM 2
#define HARD 3
#define ALL 4

#define NB_LEVELS 4

#define JUST_QUIT 5
#define RETURN_TO_MAIN_MENU 6
#define ABOUT 7

#define MINE				0x0001
#define FLAG				0x0002
#define DISCOVERED	0x0004

#define TYPE_REGULAR			0
#define TYPE_DOUBLE_CLICK	1
#define TYPE_AUTODETECT		3

#define OBJECT_MINE		0
#define OBJECT_FLAG		1

#define SPRITE_WIDTH	8
#define SPRITE_HEIGHT	8

#define WON		0
#define LOST	1
#define GO_ON 2
#define SCREEN_W (ScrRect->xy.x1+1)

#define MAX_NAME_LENGTH 9	//Max 8 caractres pour le nom
#define NB_SCORES_PER_LEVEL	5				//Max 5 scores pour chaque catgorie
#define SUB_COUNTER_SPEED 7000

#define INDEX_TIMER	1

typedef struct {
	char name[MAX_NAME_LENGTH];
	unsigned int score; 
} HI_SCORE;

typedef struct {
	short font;		//la police  utiliser
	unsigned short off_x;	//le dplacement en abscisse
	unsigned short off_y;	//le dplacement en ordonne (pour "centrer" les chiffers sur les cases)
	unsigned short fact_y;	//le facteur de multiplication pour que les lignes soient en dessous les unes des autres sans se couper ou tre trop loignes
} MINE_FONT;

//Les seules varibles globales du jeu  cause de la compatibilit 89/92;)
MINE_FONT* current_font; //la police courrante pour le jeu
MINE_FONT* m_f;						//La police du menu
//on dfinit nos deux types de police
MINE_FONT l_font = {F_6x8, 0,0,10};
MINE_FONT s_font = {F_4x6, 1,1,8};	
unsigned short offset_x;
unsigned short offset_y;

	
typedef struct {
	unsigned short flags;		//les flags du carr (voir les defines)
} MAP;

typedef	struct {
		unsigned short pos_x;								//la position du joueur dans la matrice
		unsigned short pos_y;								//en Y
		unsigned short size_x;							//la taille de la matrice
		unsigned short size_y;	
		unsigned short nb_mine;							//le nombre total de mine dans la matrice
		unsigned short nb_used_flag;				//le nombre de drapeux utiliss!
		unsigned short nb_tiles_left;				//le nombre de cases  dcouvri*r encore!
		unsigned short first_blood; 				//Rfrence  UT pour dire que le joueur a commenc  jouer!!
} GAME;

volatile unsigned short current_score;

CALLBACK void inc_current_counter(void) {
	++current_score;
}

//Prototypes de fonction
unsigned short set_level(HI_SCORE* h_s);
//La fonction qui initialise un nouveau jeu
//Retour JUST_QUIT pour quitter sinon RETURN_TO_MAIN_MENU;
short new_game(void);
MAP** create_map(unsigned short s_x, unsigned s_y, unsigned short nb_m);
void destroy_map(unsigned short s_y, MAP** m);
void play_the_game(GAME* game, MAP** map, int level, HI_SCORE* h_s);
unsigned short demine(GAME* game, MAP** map);
void window(const char* title, const char* content);
void manage_flag(GAME* game, MAP** map);
void draw_mines(GAME* game, MAP** map);
unsigned short count_objects_around(GAME* game, MAP** map, unsigned short object_type);
void draw_sprite(short posx, short posy, char* sprite, short mode);
void draw_char(short posx, short posy, char c, short mode);

HI_SCORE* load_high_score(const char* file);
HI_SCORE* create_empty_score_table(void);
void update_high_score(unsigned int comp, unsigned short level, HI_SCORE* h_s);
void save_high_score(const char* file, HI_SCORE* h_s);
void input_str(char *str,int maxlen) ;
void show_high_score(HI_SCORE* h_s, short cur_one);

//Pouah, l'horreur la reprsentation des sprites ;)
//L'astuce, c'est de faire une matrice 8*1 en binaire sur la TI pour avoir l'quivalent en HEX sans se fatiguer
static unsigned char tile[] = {0xFF,0x81,0x81,0x81,0x81,0x81,0x81,0xFF};
static unsigned char empty[] = {0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0};
static unsigned char cursor[] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
static unsigned char flag[] = {0x00,0x7C,0x7C,0x7C,0x4,0x4,0x1E,0x0};
static unsigned char wrong_flag[]  = {0x81, 0x42, 0x24, 0x18,0x18,0x24,0x42,0x81}; 	//Le drapeau tait MAL plac!
static unsigned char mine[] = {0x00,0x18,0x18,0x7E,0x7E,0x18,0x18,0x0};

//Point d'entre modifi (_main <-> main), qui a dit que les programmes TIGCC taient "portables" ;)
void _main(void)
{

	//Par dfaut sur la TI-92, on met la grande, et sur la 89, la petite :)
	if (TI89 == TRUE && V200 == FALSE)
		m_f = &s_font;
  else //TI-92 Plus - V200
  	m_f = &l_font;

	current_font = &s_font;

	while (new_game() == RETURN_TO_MAIN_MENU);

}

short new_game(void) {
	GAME* game;
	MAP** map;
	HI_SCORE* h_s;
	unsigned short level = NONE;
	unsigned short i,j;

	game = malloc(sizeof(GAME));
	if (!game)
		return JUST_QUIT;

	/* On efface l'cran */
	ClrScr();
	FontSetSys(F_8x10);
	DrawStr (0, 0, "Dmineur "VERSION, A_REPLACE);
	FontSetSys(m_f->font);
	DrawStr (0, 2*m_f->fact_y, "1) Facile", A_NORMAL);
	DrawStr (0, 3*m_f->fact_y, "2) Moyen", A_NORMAL);
	DrawStr (0, 4*m_f->fact_y, "3) Difficile", A_NORMAL);
	DrawStr (0, 5*m_f->fact_y, "4) !!ULTIME!!", A_NORMAL);
	DrawStr (SCREEN_W/2, 2*m_f->fact_y, "F1) Dminer", A_NORMAL);
	if (TI89 == TRUE && V200 == FALSE)
		DrawStr (SCREEN_W/2, 3*m_f->fact_y, "F2) Marquer la zone", A_NORMAL);
	else
		DrawStr (SCREEN_W/2, 3*m_f->fact_y, "F5) Marquer la zone", A_NORMAL);

	DrawStr (SCREEN_W/2, 4*m_f->fact_y, "Esc) Quitter", A_NORMAL);	
	DrawStr (SCREEN_W/2, 5*m_f->fact_y, "Apps) A propos...", A_NORMAL);

	DrawStr (0, 7*m_f->fact_y, "F3) Nombre de drapeaux restants", A_NORMAL);	
	DrawStr (0, 8*m_f->fact_y, "F4) High Scores", A_NORMAL);	
	DrawStr (0, 9*m_f->fact_y, "F6) Eteindre la calculatrice!", A_NORMAL);	

	h_s = load_high_score(SAVE_NAME);
	if (!h_s) {
		free(game);
		return JUST_QUIT;
	}
		
	/* Maintenant, on dessine les parties variables dpendantes de l'utilisateur!*/
	while ((level = set_level(h_s)) == NONE)
		;

	if (level == JUST_QUIT) {
		free(game);
		free(h_s);
		return JUST_QUIT;
	}
		
	switch (level) {
		case EASY:
		  game->size_x = 10;
		  game->size_y = 10;
		  game->nb_mine = 16;
			break;
		case MEDIUM:
		  game->size_x = 15;
		  game->size_y = 12;
		  game->nb_mine = 32;
			break;
		case HARD:
			if (TI89 == TRUE && V200 == FALSE) {
			  game->size_x = 20;
			  game->size_y = 12;
		  	game->nb_mine = 45;
		 	}
		 	else {
			  game->size_x = 20;
			  game->size_y = 16;
		  	game->nb_mine = 60;
		 	}
			break;
		case ALL:
			if (TI89 == TRUE && V200 == FALSE) {
			  game->size_x = 20;
			  game->size_y = 12;
			  game->nb_mine = 55;
			}
			else {
			  game->size_x = 30;
			  game->size_y = 16;
			  game->nb_mine = 100;
			}
			break;
		default:
			free(game);
			free(h_s);
			return JUST_QUIT;
	}
	
	offset_x = ((ScrRect->xy.x1+1) - game->size_x*SPRITE_WIDTH)/2;
	offset_y = ((ScrRect->xy.y1+9) - game->size_y*SPRITE_HEIGHT)/2;

	//On cre une matrice
	if ((map = create_map(game->size_x, game->size_y, game->nb_mine)) == NULL)
		return JUST_QUIT;

	//On intialise le reste des variables
	game->pos_x = 0;
	game->pos_y = 0;
	game->nb_used_flag = 0;
	game->nb_tiles_left = (game->size_x)*(game->size_y) - game->nb_mine;
	game->first_blood = FALSE;

	/* On efface l'cran */
	ClrScr();

	//On dessine la carte!
	for (i = 0; i < game->size_y; ++i) {
			for (j = 0; j < game->size_x; ++j) {
				draw_sprite(j, i, tile, SPRT_XOR);
			}
	}
	
	//ON JOUE LE JEU!
	play_the_game(game, map, level, h_s);
	if (game->first_blood == TRUE)
	  OSVFreeTimer(INDEX_TIMER);
  
	destroy_map(game->size_y, map);
	free(game);
	free(h_s);
	return RETURN_TO_MAIN_MENU;
}

void play_the_game(GAME* game, MAP** map, int level, HI_SCORE* h_s) {
	unsigned short key;
	char buf[32];
	unsigned short final_score = 0;
		
	//on dssine le curseur  l'origine!
	draw_sprite(game->pos_x, game->pos_y, cursor, SPRT_XOR);

	//LA BOUCLE PRINCIPALE DU JEU
	
	current_score = 0;
	
	while(1) {
	
		while (kbhit() == 0) {
			;
		}
	
		key = ngetchx();
		if (key == 264)
			return;	//on revient au dpart (presque)!

		//on efface le curseur!
		draw_sprite(game->pos_x, game->pos_y, cursor, SPRT_XOR);
		if (key == 264)
				return;
		else if (key == KEY_LEFT) {
			if ((short)game->pos_x - 1 < 0)
				game->pos_x = game->size_x-1;
			else
				--game->pos_x;
		}
		else if (key == KEY_RIGHT) {
			++game->pos_x;
			if (game->pos_x >= game->size_x)
				game->pos_x = 0;
		}
		else if (key == KEY_DOWN) {
			++game->pos_y;
			if (game->pos_y >= game->size_y)
				game->pos_y = 0;
		}
		else if (key == KEY_UP) {
			if ((short)game->pos_y -1 < 0)
				game->pos_y = game->size_y-1;
			else
				--game->pos_y;
		}
		else if (key == 268) { //F1
			//On ne veut pas lancer le truc rcursif si on appuie sur un drapeau!
			if (!(map[game->pos_y][game->pos_x].flags & FLAG)) {
				switch(demine(game, map)) {
					case LOST:
						final_score = current_score;
						//On dessine toutes les mines
						draw_mines(game,map);
						sprintf(buf, "Tu as perdu ! (%d s)", final_score);						
						window("Hlas!", buf);
						ngetchx();					
						return;
					case WON:
						final_score = current_score;
						sprintf(buf, "Score : %d ", final_score);
						window("Pas mal!", buf);
						update_high_score(final_score, level, h_s);
						ngetchx();					
						return;
					default:
						if (game->first_blood == FALSE) {
							OSVRegisterTimer (INDEX_TIMER, 20, inc_current_counter);
							game->first_blood = TRUE;
						}
						break;
				}
			}
	  }
	  else if (key == 269 && TI89 == TRUE && V200 == FALSE) { //F2
				manage_flag(game, map);
	  }
	  else if (key == 272 && TI89 == FALSE && V200 == FALSE) {  //F5
			manage_flag(game, map);
	  }
	  else if (key == 270) {  //F3
	  	sprintf(buf, "Etat : %d / %d (%d s)", game->nb_used_flag, game->nb_mine, current_score);
	  	window("Drapeaux / Mines", buf);
	  }
	  else if (key == 273) {  //F6
	  	off();
	  }
		//on reaffiche le sprite		
		draw_sprite(game->pos_x, game->pos_y, cursor, SPRT_XOR);

			
	}

	return;
}

MAP** create_map(unsigned short s_x, unsigned s_y, unsigned short nb_m) {
	MAP** m;
	unsigned short i,j;

	m = malloc(s_y*sizeof(MAP*));
	if (!m)
		return NULL;
	
	for (i = 0; i < s_y; ++i) {
		m[i] = malloc(sizeof(MAP)*s_x);
		if (!m[i])
			return NULL;
	}

	for (i = 0; i < s_y; ++i) {
		for (j = 0; j < s_x; ++j) {
			m[i][j].flags = 0;
		}	
	}
	
	while (nb_m > 0) {
		i = rand()%s_y;
		j = rand()%s_x;
		if (m[i][j].flags & MINE)
			continue;
		
		m[i][j].flags |= MINE;
		--nb_m;
	}
	
	return m;
}

void destroy_map(unsigned short s_y, MAP** m) {
	unsigned short i;
	for (i = 0; i < s_y; ++i)
		free(m[i]);
	free(m);
	return;
}



unsigned short set_level(HI_SCORE* h_s) {
	static unsigned long i = 0;	

	while(kbhit() == 0)
		++i;
	
	srand(i++); //i++ : au cas ou on ait pas le temps d'incrmenter i en haut :)
	
	//Ici, ngetchx peut traiter le resultat de kbhit() qui est != 0
  switch (  ngetchx()) {
  	case 49:
  	  return EASY;
  	case 50:
  		return MEDIUM;
  	case 51:
  		return HARD;
  	case 52:
  		return ALL;
  	case 264:
  	  return JUST_QUIT;
  	case 265:
  		window("Dmineur "VERSION" "VERSION_TIME, "Par Emeric Poupon (ITM)");
  		return NONE;
		case 271:
			show_high_score(h_s,-1);  		
			return NONE;
		default:
			return NONE;
  }
}
	
unsigned short demine(GAME* game, MAP** map) {
	short i,j;
	unsigned short nb_mines_around = 0;
	char nb_mine_str[2];
	unsigned short type;

	
	nb_mines_around = count_objects_around(game, map, OBJECT_MINE);

	if (map[game->pos_y][game->pos_x].flags & DISCOVERED)
		type = TYPE_DOUBLE_CLICK;
	else
		type = TYPE_REGULAR;

	switch (type) {
		case TYPE_REGULAR:
			if (map[game->pos_y][game->pos_x].flags & MINE)
				return LOST;

			if (map[game->pos_y][game->pos_x].flags & DISCOVERED)
				return GO_ON;

			map[game->pos_y][game->pos_x].flags |= DISCOVERED;
			--game->nb_tiles_left;
	
			//on affiche un carr blanc! Utile aussi pour mettre un chiffre proprement dessus
			draw_sprite(game->pos_x, game->pos_y, empty, SPRT_AND);

			if (nb_mines_around == 0) {
				for (i = -1; i <= 1; ++i) {
					if ((signed short)game->pos_y + i < 0 || game->pos_y +i >= game->size_y)
						continue;
					for (j = -1; j <= 1; ++j) {
						if ((signed short)game->pos_x + j < 0 || game->pos_x + j >= game->size_x)
							continue;
						if (!(map[game->pos_y+i][game->pos_x+j].flags & DISCOVERED)) {
							//si la case  explorer  DEJA un drapeau (a veut dire que le gars ne sait pas jouer mais bon, on va tre gentil, on va lui enlever ;)
							if ((map[game->pos_y+i][game->pos_x+j].flags & FLAG)) {
								map[game->pos_y+i][game->pos_x+j].flags ^= FLAG;
								--game->nb_used_flag;
								//on retire le drapeau -> pas la peine, le carr blanc est affich avant cette boucle :)
							}
							game->pos_x += j;
							game->pos_y += i;
							demine(game, map);
							game->pos_x -= j;
							game->pos_y -= i;
						}
					}
				}
			}
			else {
				FontSetSys(current_font->font);
				sprintf(nb_mine_str, "%d", nb_mines_around);
				//On affiche le nombre de mines!	
				draw_char(game->pos_x, game->pos_y, nb_mine_str[0], A_REPLACE);
				
			}
			break;

		case TYPE_DOUBLE_CLICK:
			if (!(map[game->pos_y][game->pos_x].flags & DISCOVERED))
				return GO_ON;
				
			if (nb_mines_around != count_objects_around(game, map, OBJECT_FLAG))
				return GO_ON;

			for (i = -1; i <= 1; ++i) {
				if ((signed short)game->pos_y + i < 0 || game->pos_y +i >= game->size_y)
					continue;
				for (j = -1; j <= 1; ++j) {
					if ((signed short)game->pos_x + j < 0 || game->pos_x + j >= game->size_x)
						continue;
					if (!(map[game->pos_y+i][game->pos_x+j].flags & DISCOVERED) && !(map[game->pos_y+i][game->pos_x+j].flags & FLAG)) {
						//si la case  explorer n'a pas de drapeau, on va pouvoir faire des trucs!
						if (map[game->pos_y+i][game->pos_x+j].flags & MINE)
							return LOST;
						else {
							game->pos_x += j;
							game->pos_y += i;

							nb_mines_around = count_objects_around(game, map, OBJECT_MINE);
							if (nb_mines_around == 0)
								demine(game, map);
							else {
								map[game->pos_y][game->pos_x].flags |= DISCOVERED;
								--game->nb_tiles_left;

								FontSetSys(current_font->font);

								sprintf(nb_mine_str, "%d", nb_mines_around);
								//On affiche le nombre de mines!	
								//on affiche un carr blanc! Utile aussi pour mettre un chiffre proprement dessus
								draw_sprite(game->pos_x, game->pos_y, empty, SPRT_AND);
								draw_char(game->pos_x, game->pos_y, nb_mine_str[0], A_REPLACE);
							}

							//On recentre
							game->pos_x -= j;
							game->pos_y -= i;
						}							
					}
				}
			}
				
			break;
	}


	//Si le gars a dcouvert la dernire case :)
	if (game->nb_tiles_left == 0)
		return WON;
	
	return GO_ON;
}

void window(const char* title, const char* content) {
	short f = FontGetSys();
	WIN_RECT w_r;
	unsigned short sx = strlen(content);
	WINDOW	w;
	w_r.x0 = 20;
	w_r.y0 = 20;
	w_r.x1 = sx*8;
	w_r.y1 = 50;
	
  WinOpen(&w,&w_r,WF_SAVE_SCR|WF_ROUNDEDBORDER|WF_TITLE, title);
  WinFont(&w,F_6x8);
  WinActivate(&w);
  WinStrXY(&w,4,4,(char*)content);
	ngetchx();
	WinClose(&w);
	FontSetSys(f);
}

void manage_flag(GAME* game, MAP** map) {
	if (map[game->pos_y][game->pos_x].flags & DISCOVERED)
		return;

	if (map[game->pos_y][game->pos_x].flags & FLAG) {
		map[game->pos_y][game->pos_x].flags ^= FLAG;
		--game->nb_used_flag;
		//on affiche un carr blanc! On pourra dessiner un drapeau ou une case proprement ainsi!
		draw_sprite(game->pos_x, game->pos_y, empty, SPRT_AND);
		//on retire le drapeau
		draw_sprite(game->pos_x, game->pos_y, tile, SPRT_XOR);		
		
		return;
	}
	else if (game->nb_used_flag == game->nb_mine)
		return;

	//Ici, on est sur qu'on a des drapeaux de libre!
	++game->nb_used_flag;
	map[game->pos_y][game->pos_x].flags |= FLAG;
	//on affiche un carr blanc! On pourra dessiner un drapeau ou une case proprement ainsi!
	draw_sprite(game->pos_x, game->pos_y, empty, SPRT_AND);	
	//on affiche le drapeau
		draw_sprite(game->pos_x, game->pos_y, flag, SPRT_XOR);			
	return;
}

void draw_mines(GAME* game, MAP** map) {
	unsigned short i,j;
	for (i = 0; i < game->size_y; ++i) {
		for (j = 0; j < game->size_x; ++j) {
			if (map[i][j].flags & MINE && !(map[i][j].flags & FLAG) ) {
				draw_sprite(j, i, empty, SPRT_AND);		
				draw_sprite(j, i, mine, SPRT_XOR);		
			}
			else if (map[i][j].flags & FLAG && !(map[i][j].flags & MINE)) {
				draw_sprite(j, i, empty, SPRT_AND);						
				draw_sprite(j, i, wrong_flag, SPRT_XOR);						
			}
		}
	}
}


void save_high_score(const char* file, HI_SCORE* h_s) {
	FILE* fp;
	fp = fopen(file, "w");
	if (!fp)
		return;  //on quitte pour eviter les dgats!
	
	fwrite(h_s, sizeof(HI_SCORE), NB_SCORES_PER_LEVEL*NB_LEVELS, fp); 
	fclose(fp);	
}

HI_SCORE* load_high_score(const char* file) {
	FILE* fp;
	HI_SCORE* h_s;
	
	h_s = create_empty_score_table();
	if (!h_s)
		return NULL;

	fp = fopen(file, "r");
	if (!fp)
		return h_s;  //on retourne une table vierge!

	//On lit le contenu du fichier
	fread(h_s, sizeof(HI_SCORE), NB_SCORES_PER_LEVEL*NB_LEVELS, fp);

	fclose(fp);
	return h_s;
}

HI_SCORE* create_empty_score_table(void) {
	unsigned short i;
	HI_SCORE* h_s;	

	h_s = malloc(sizeof(HI_SCORE)*NB_LEVELS*NB_SCORES_PER_LEVEL);
	if (!h_s)
		return NULL;
	
	for (i = 0; i < NB_LEVELS*NB_SCORES_PER_LEVEL; ++i) {
		strcpy(h_s[i].name, "????????");
		h_s[i].score = 9999;
	}
	
	return h_s;
}

void update_high_score(unsigned int score, unsigned short level, HI_SCORE* h_s) {
	short i,j;

	for (i = (level-1)*NB_SCORES_PER_LEVEL; (unsigned short)i < level*NB_SCORES_PER_LEVEL; ++i) {
		if (score < h_s[i].score) {
		  for (j = level*NB_SCORES_PER_LEVEL-1; j > i; --j) {
		  	 strcpy(h_s[j].name, h_s[j-1].name);
		  	 h_s[j].score = h_s[j-1].score;
		  }
			//On rajoute son score:
			
			input_str(h_s[i].name, MAX_NAME_LENGTH-1);
			h_s[i].score = score;
			save_high_score(SAVE_NAME, h_s);
			show_high_score(h_s, i);			
			return;
		}
	}

	return;
}

void input_str(char *str,int maxlen) {
	short key = 0, i = 0;
	short f = FontGetSys();
	WIN_RECT w_r;
	WINDOW	w;
	w_r.x0 = 20;
	w_r.y0 = 20;
	w_r.x1 = 120;
	w_r.y1 = 50;
	
  WinOpen(&w,&w_r,WF_SAVE_SCR|WF_ROUNDEDBORDER|WF_TITLE, "NEW HI-SCORE!");
  WinFont(&w,F_6x8);
  WinActivate(&w);

	while (key != 13) {
		WinClr(&w);
	  WinStrXY(&w,4,4, (char*)"Nom :");		str[i] = '_';
		str[i+1] = ' ';
		str[i+2] = 0;
		WinStrXY (&w, 40, 4, str);
		key = ngetchx();
		if (key >= ' ' && key <= '~' && i < maxlen)
			str[i++] = key;
		if (key == 257 && i) 
			i--;
    }
	str[i] = 0;

	WinClose(&w);
	FontSetSys(f);
}

void show_high_score(HI_SCORE* h_s, short cur_one) {
	unsigned short i;
	char buf[10];
	WIN_RECT w_r;
	WINDOW	w;
	short f = FontGetSys();
		
	w_r.x0 = 2;
	w_r.y0 = 4;
	w_r.x1 = 130;
	w_r.y1 = 88;
  WinOpen(&w,&w_r,WF_SAVE_SCR|WF_ROUNDEDBORDER);
  WinFont(&w,F_4x6);
  WinActivate(&w);

	WinStrXY (&w, 8, 2, (char*)"Facile");
	for (i = 0; i < NB_SCORES_PER_LEVEL*NB_LEVELS/4; ++i) {
		WinStrXY (&w, 4, 2+(i+1)*6, h_s[i].name);
		sprintf(buf, "%d", h_s[i].score);
		WinStrXY (&w, 40, 2+(i+1)*6, buf);
		if (cur_one == (short)i)
			WinStrXY (&w, 0, 2+(i+1)*6, (char*)"*");				
	}
	WinStrXY (&w, 8, 2+6*(NB_SCORES_PER_LEVEL*NB_LEVELS/4+2), (char*)"Moyen");	
	for (i = NB_SCORES_PER_LEVEL*NB_LEVELS/4; i < NB_SCORES_PER_LEVEL*NB_LEVELS/2; ++i) {
		WinStrXY (&w, 4, 2+(i+3)*6, h_s[i].name);
		sprintf(buf, "%d", h_s[i].score);
		WinStrXY (&w, 40, 2+(i+3)*6, buf);
		if (cur_one == (short)i)		
			WinStrXY (&w, 0, 2+(i+3)*6, (char*)"*");						
	}

	WinStrXY (&w, 74, 2, (char*)"Difficile");
	for (i = NB_SCORES_PER_LEVEL*NB_LEVELS/2; i < 3*NB_SCORES_PER_LEVEL*NB_LEVELS/4; ++i) {
		WinStrXY (&w, 70, 2+(i-NB_SCORES_PER_LEVEL*NB_LEVELS/2+1)*6, h_s[i].name);
		sprintf(buf, "%d", h_s[i].score);
		WinStrXY (&w, 106, 2+(i-NB_SCORES_PER_LEVEL*NB_LEVELS/2+1)*6, buf);		
		if (cur_one == (short)i)		
			WinStrXY (&w, 66, 2+(i-NB_SCORES_PER_LEVEL*NB_LEVELS/2+1)*6, (char*)"*");						
	}
	WinStrXY (&w, 74, 2+6*(NB_SCORES_PER_LEVEL*NB_LEVELS/4+2), (char*)"!!ULTIME!!");	
	for (i = 3*NB_SCORES_PER_LEVEL*NB_LEVELS/4; i < NB_SCORES_PER_LEVEL*NB_LEVELS; ++i) {
		WinStrXY (&w, 70, 2+(i-NB_SCORES_PER_LEVEL*NB_LEVELS/2+3)*6, h_s[i].name);
		sprintf(buf, "%d", h_s[i].score);
		WinStrXY (&w, 106, 2+(i-NB_SCORES_PER_LEVEL*NB_LEVELS/2+3)*6, buf);
		if (cur_one == (short)i)		
			WinStrXY (&w, 66, 2+(i-NB_SCORES_PER_LEVEL*NB_LEVELS/2+3)*6, (char*)"*");								
	}
	
	ngetchx();
	WinClose(&w);
	FontSetSys(f);
}

unsigned short count_objects_around(GAME* game, MAP** map, unsigned short object_type) {
	unsigned short nb_objects_around = 0;
	short i, j;

	for (i = -1; i <= 1; ++i) {
		if ((signed short)game->pos_y + i < 0 || game->pos_y +i >= game->size_y)
			continue;
		for (j = -1; j <= 1; ++j) {
			if ((signed short)game->pos_x + j < 0 || game->pos_x +j >= game->size_x)
				continue;
			if (map[game->pos_y+i][game->pos_x+j].flags & MINE && object_type == OBJECT_MINE)
				++nb_objects_around;
			else if (map[game->pos_y+i][game->pos_x+j].flags & FLAG && object_type == OBJECT_FLAG)
				++nb_objects_around;
		}
	}
	return nb_objects_around;
}
	//Pour pas se faire chier, les dcalages pour centre l'affichage

void draw_sprite(short posx, short posy, char* sprite, short mode) {
	Sprite8(posx * SPRITE_WIDTH+offset_x, posy * SPRITE_HEIGHT+offset_y, SPRITE_HEIGHT, sprite, LCD_MEM, mode);	
}


void draw_char(short posx, short posy, char c, short mode) {
	DrawChar(posx * SPRITE_WIDTH + current_font->off_x + offset_x, posy * SPRITE_HEIGHT+offset_y+current_font->off_y, c, mode);
//DrawChar (game->pos_x * SPRITE_WIDTH +current_font->off_x+offset_x, game->pos_y * SPRITE_HEIGHT+current_font->off_y+offset_y, nb_mine_str[0], A_REPLACE);		
}